@@ -4,7 +4,7 @@ use warnings;
 
 BEGIN {
   $ntheory::AUTHORITY = 'cpan:DANAJ';
-  $ntheory::VERSION = '0.61';
+  $ntheory::VERSION = '0.62';
 }
 
 BEGIN {
@@ -35,6 +35,10 @@ See L<Math::Prime::Util> for complete documentation.
 
 =head1 QUICK REFERENCE
 
+Tags:
+  :all         to import almost all functions
+  :rand        to import rand, srand, irand, irand64
+
 =head2 PRIMALITY
 
   is_prob_prime(n)                    primality test (BPSW)
@@ -90,7 +94,13 @@ See L<Math::Prime::Util> for complete documentation.
   nth_twin_prime_approx(n)            fast approximate nth twin prime
   ramanujan_prime_count(n)            count of Ramanujan primes <= n
   ramanujan_prime_count(start, end)   count of Ramanujan primes in range
+  ramanujan_prime_count_lower(n)      fast lower bound for Ramanujan count
+  ramanujan_prime_count_upper(n)      fast upper bound for Ramanujan count
+  ramanujan_prime_count_approx(n)     fast approximate Ramanujan count
   nth_ramanujan_prime(n)              the nth Ramanujan prime (Rn)
+  nth_ramanujan_prime_lower(n)        fast lower bound for Rn
+  nth_ramanujan_prime_upper(n)        fast upper bound for Rn
+  nth_ramanujan_prime_approx(n)       fast approximate Rn
   legendre_phi(n,a)                   # below n not div by first a primes
   inverse_li(n)                       integer inverse logarithmic integral
   prime_precalc(n)                    precalculate primes to n
@@ -121,6 +131,19 @@ See L<Math::Prime::Util> for complete documentation.
   prime_iterator                      returns a simple prime iterator
   prime_iterator_object               returns a prime iterator object
 
+=head2 RANDOM NUMBERS
+
+  irand                               random 32-bit integer
+  irand64                             random 64-bit integer
+  drand([limit])                      random NV in [0,1) or [0,limit)
+  random_bytes(n)                     string with n random bytes
+  entropy_bytes(n)                    string with n entropy-source bytes
+  urandomb(n)                         random integer less than 2^n
+  urandomm(n)                         random integer less than n
+  csrand(data)                        seed the CSPRNG with binary data
+  srand([seed])                       simple seed (exported with :rand)
+  rand([limit])                       alias for drand (exported with :rand)
+
 =head2 RANDOM PRIMES
 
   random_prime([start,] end)          random prime in a range
@@ -234,7 +257,7 @@ See L<Math::Prime::Util> for complete documentation.
 
 =head1 COPYRIGHT
 
-Copyright 2011-2016 by Dana Jacobsen E<lt>dana@acm.orgE<gt>
+Copyright 2011-2017 by Dana Jacobsen E<lt>dana@acm.orgE<gt>
 
 This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
 
